# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import click

import zhmcclient
from .zhmccli import cli
from ._helper import print_properties, print_resources, abort_if_false


@cli.group('lpar')
def lpar_group():
    """Command group for managing LPARs."""


@lpar_group.command('list')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.pass_obj
def lpar_list(cmd_ctx, cpc_name):
    """List the LPARs in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_lpar_list(cmd_ctx, cpc_name))


@lpar_group.command('show')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('LPAR-NAME', type=str, metavar='LPAR-NAME')
@click.pass_obj
def lpar_show(cmd_ctx, cpc_name, lpar_name):
    """Show details of an LPAR in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_lpar_show(cmd_ctx, cpc_name, lpar_name))


@lpar_group.command('activate')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('LPAR-NAME', type=str, metavar='LPAR-NAME')
@click.pass_obj
def lpar_activate(cmd_ctx, cpc_name, lpar_name):
    """Activate an LPAR in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_lpar_activate(cmd_ctx, cpc_name,
                                                  lpar_name))


@lpar_group.command('deactivate')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('LPAR-NAME', type=str, metavar='LPAR-NAME')
@click.option('--yes', is_flag=True, callback=abort_if_false,
              expose_value=False,
              help='Skip prompt to confirm deactivation of the LPAR.',
              prompt='Are you sure you want to deactivate the LPAR ?')
@click.pass_obj
def lpar_deactivate(cmd_ctx, cpc_name, lpar_name):
    """Deactivate an LPAR in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_lpar_deactivate(cmd_ctx, cpc_name,
                                                    lpar_name))


@lpar_group.command('load')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('LPAR-NAME', type=str, metavar='LPAR-NAME')
@click.argument('LOAD-ADDRESS', type=str, metavar='LOAD-ADDRESS')
@click.pass_obj
def lpar_load(cmd_ctx, cpc_name, lpar_name, load_address):
    """Load an LPAR in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_lpar_load(cmd_ctx, cpc_name, lpar_name,
                                              load_address))


def _find_cpc(client, cpc_name):
    try:
        cpc = client.cpcs.find(name=cpc_name)
    except zhmcclient.NotFound:
        raise click.ClickException("Could not find CPC %s on HMC %s." %
                                   (cpc_name, client.session.host))
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    return cpc


def _find_lpar(client, cpc_name, lpar_name):
    cpc = _find_cpc(client, cpc_name)
    if cpc.dpm_enabled:
        raise click.ClickException("CPC %s is in DPM mode." % cpc_name)
    try:
        lpar = cpc.lpars.find(name=lpar_name)
    except zhmcclient.NotFound:
        raise click.ClickException("Could not find LPAR %s in CPC %s." %
                                   (lpar_name, cpc_name))
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    return lpar


def cmd_lpar_list(cmd_ctx, cpc_name):
    """
    List the LPARs in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    cpc = _find_cpc(client, cpc_name)
    if cpc.dpm_enabled:
        raise click.ClickException("CPC %s is in DPM mode." % cpc_name)
    try:
        lpars = cpc.lpars.list()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    print_resources(lpars, cmd_ctx.output_format)


def cmd_lpar_show(cmd_ctx, cpc_name, lpar_name):
    """
    Show the details of an LPAR in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    lpar = _find_lpar(client, cpc_name, lpar_name)
    try:
        lpar.pull_full_properties()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    skip_list = ('program-status-word-information')
    print_properties(lpar.properties, cmd_ctx.output_format, skip_list)


def cmd_lpar_activate(cmd_ctx, cpc_name, lpar_name):
    """
    Activate an LPAR in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    lpar = _find_lpar(client, cpc_name, lpar_name)
    try:
        lpar.activate(wait_for_completion=True)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    click.echo('Activation of LPAR %s is complete.' % lpar_name)


def cmd_lpar_deactivate(cmd_ctx, cpc_name, lpar_name):
    """
    Deactivate an LPAR in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    lpar = _find_lpar(client, cpc_name, lpar_name)
    try:
        lpar.deactivate(wait_for_completion=True)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    click.echo('Deactivation of LPAR %s is complete.' % lpar_name)


def cmd_lpar_load(cmd_ctx, cpc_name, lpar_name, load_address):
    """
    Load an LPAR in a CPC from a load address.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    lpar = _find_lpar(client, cpc_name, lpar_name)
    try:
        lpar.load(load_address, wait_for_completion=True)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    click.echo('Loading of LPAR %s is complete.' % lpar_name)



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example 8: Using the Adapter, Port, Virtual Switch, NIC, HBA
           and Virtual Function interface.
"""

import sys
import yaml
import requests.packages.urllib3
import time

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

example8 = examples.get("example8", None)
if example8 is None:
    print("example8 not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = example8.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = example8["hmc"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

print(__doc__)

try:
    print("Using HMC %s with userid %s ..." % (hmc, userid))
    session = zhmcclient.Session(hmc, userid, password)
    cl = zhmcclient.Client(session)

    timestats = example8.get("timestats", None)
    if timestats:
        session.time_stats_keeper.enable()

    print("Listing CPCs ...")
    cpcs = cl.cpcs.list()
    for cpc in cpcs:
        print(cpc)
        print("\tListing Adapters for %s ..." % cpc.properties['name'])
        adapters = cpc.adapters.list()
        for i, adapter in enumerate(adapters):
            print('\t' + str(adapter))
            ports = adapter.ports.list(full_properties=False)
            for p, port in enumerate(ports):
                if p == 0:
                    print("\t\tListing Ports for %s ..." % adapter.properties['name'])
#                port.pull_full_properties()
                print('\t\t' + str(port))
        print("\tListing Virtual Switches for %s ..." % cpc.properties['name'])
        vswitches = cpc.vswitches.list()
        for i, vswitch in enumerate(vswitches):
            print('\t' + str(vswitch))
        print("\tListing Partitions for %s ..." % cpc.properties['name'])
        partitions = cpc.partitions.list()
        for i, partition in enumerate(partitions):
            print('\t' + str(partition))
            nics = partition.nics.list(full_properties=False)
            for j, nic in enumerate(nics):
                if j == 0:
                    print("\t\tListing NICs for %s ..." % partition.properties['name'])
                print('\t\t' + str(nic))

            hbas = partition.hbas.list(full_properties=False)
            for j, hba in enumerate(hbas):
                if j == 0:
                    print("\t\tListing HBAs for %s ..." % partition.properties['name'])
                print('\t\t' + str(hba))
#                hba.pull_full_properties()
#                print('\t\t' + str(hba.properties))
            vfs = partition.virtual_functions.list(full_properties=False)
            for k, vf in enumerate(vfs):
                if k == 0:
                    print("\t\tListing Virtual Functions for %s ..." % partition.properties['name'])
                print('\t\t' + str(vf))
#                vf.pull_full_properties()
#                print('\t\t' + str(vf.properties))

    print("Logging off ...")
    session.logoff()

    if timestats:
        print(session.time_stats_keeper)

    print("Done.")

except zhmcclient.Error as exc:
    print("%s: %s" % (exc.__class__.__name__, exc))
    sys.exit(1)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
An :term:`Adapter` is a physical adapter card (e.g. OSA-Express adapter,
Crypto adapter) or a logical adapter (e.g. HiperSockets switch).

Adapter resources are contained in :term:`CPC` resources.

Adapters only exist in CPCs that are in DPM mode.

There are four types of Adapters:

1. Network Adapters:
   Network adapters enable communication through different networking
   transport protocols. These network adapters are OSA-Express,
   HiperSockets and RoCE-Express.
   DPM automatically discovers OSA-Express and RoCE-Express adapters
   because they are physical cards that are installed on the CPC.
   In contrast, HiperSockets are logical adapters and must be
   created and configured by an administrator using the 'Create Hipersocket'
   operation (see create_hipersocket()).
   Network Interface Cards (NICs) provide a partition with access to networks.
   Each NIC represents a unique connection between the partition
   and a specific network adapter.

2. Storage Adapters:
   Fibre Channel connections provide high-speed connections between CPCs
   and storage devices.
   DPM automatically discovers any storage adapters installed on the CPC.
   Host bus adapters (HBAs) provide a partition with access to external
   storage area networks (SANs) and devices that are connected to a CPC.
   Each HBA represents a unique connection between the partition
   and a specific storage adapter.

3. Accelerator Adapters:
   Accelerator adapters provide specialized functions to
   improve performance or use of computer resource like the IBM System z
   Enterprise Data Compression (zEDC) feature.
   DPM automatically discovers accelerators that are installed on the CPC.
   An accelerator virtual function provides a partition with access
   to zEDC features that are installed on a CPC.
   Each virtual function represents a unique connection between
   the partition and a physical feature card.

4. Crypto Adapters:
   Crypto adapters provide cryptographic processing functions.
   DPM automatically discovers cryptographic features that are installed
   on the CPC.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource
from ._port import PortManager

__all__ = ['AdapterManager', 'Adapter']


class AdapterManager(BaseManager):
    """
    Manager providing access to the :term:`Adapters <Adapter>` in a particular
    :term:`CPC`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible as properties in higher level resources (in this case, the
    :class:`~zhmcclient.Cpc` object).
    """

    def __init__(self, cpc):
        # This function should not go into the docs.
        # Parameters:
        #   cpc (:class:`~zhmcclient.Cpc`):
        #     CPC defining the scope for this manager.
        super(AdapterManager, self).__init__(cpc)

    @property
    def cpc(self):
        """
        :class:`~zhmcclient.Cpc`: :term:`CPC` defining the scope for this
        manager.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        List the Adapters in this CPC.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.Adapter` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        adapters_res = self.session.get(cpc_uri + '/adapters')
        adapter_list = []
        if adapters_res:
            adapter_items = adapters_res['adapters']
            for adapter_props in adapter_items:
                adapter = Adapter(self, adapter_props['object-uri'],
                                  adapter_props)
                if full_properties:
                    adapter.pull_full_properties()
                adapter_list.append(adapter)
        return adapter_list

    def create_hipersocket(self, properties):
        """
        Create and configure a HiperSockets Adapter in this CPC.

        Parameters:

          properties (dict): Initial property values.
            Allowable properties are defined in section 'Request body contents'
            in section 'Create Hipersocket' in the :term:`HMC API` book.

        Returns:

          Adapter:
            The resource object for the new HiperSockets Adapter.
            The object will have its 'object-uri' property set as returned by
            the HMC, and will also have the input properties set.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        result = self.session.post(cpc_uri + '/adapters', body=properties)
        # There should not be overlaps, but just in case there are, the
        # returned props should overwrite the input props:
        props = properties.copy()
        props.update(result)
        return Adapter(self, props['object-uri'], props)


class Adapter(BaseResource):
    """
    Representation of an :term:`Adapter`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    For the properties of an Adapter, see section 'Data model' in section
    'Adapter object' in the :term:`HMC API` book.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.AdapterManager`).
    """

    def __init__(self, manager, uri, properties):
        # This function should not go into the docs.
        # Parameters:
        #   manager (:class:`~zhmcclient.AdapterManager`):
        #     Manager for this Adapter.
        #   uri (string):
        #     Canonical URI path of this Adapter.
        #   properties (dict):
        #     Properties to be set for this Adapter.
        #     See initialization of :class:`~zhmcclient.BaseResource` for
        #     details.
        assert isinstance(manager, AdapterManager)
        super(Adapter, self).__init__(manager, uri, properties)
        self._ports = None

    @property
    def ports(self):
        """
        :class:`~zhmcclient.PortManager`: Access to the :term:`Ports <Port>` of
        this Adapter.
        """
        # We do here some lazy loading.
        if not self._ports:
            self._ports = PortManager(self)
        return self._ports

    def delete(self):
        """
        Delete this Adapter.

        The Adapter must be a HiperSockets Adapter.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        adapter_uri = self.get_property('object-uri')
        self.manager.session.delete(adapter_uri)

    def update_properties(self, properties):
        """
        Update writeable properties of this Adapter.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model' in section 'Adapter object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        adapter_uri = self.get_property('object-uri')
        self.manager.session.post(adapter_uri, body=properties)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`Port` is a physical connector port (jack) of an :term:`Adapter`.

Port resources are contained in Adapter resources.

Ports only exist in :term:`CPCs <CPC>` that are in DPM mode.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource

__all__ = ['PortManager', 'Port']


class PortManager(BaseManager):
    """
    Manager providing access to the :term:`Ports <Port>` of a particular
    :term:`Adapter`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible as properties in higher level resources (in this case, the
    :class:`~zhmcclient.Adapter` object).
    """

    def __init__(self, adapter):
        # This function should not go into the docs.
        # Parameters:
        #   adapter (:class:`~zhmcclient.Adapter`):
        #     Adapter defining the scope for this manager.
        super(PortManager, self).__init__(adapter)

    @property
    def adapter(self):
        """
        :class:`~zhmcclient.Adapter`: :term:`Adapter` defining the scope for
        this manager.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        List the Ports of this Adapter.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.Port` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        port_list = []
        storage_family = ('ficon')
        network_family = ('osa', 'roce', 'hipersockets')
        if self.adapter.get_property('adapter-family') in storage_family:
            ports_res = self.adapter.get_property('storage-port-uris')
        elif self.adapter.get_property('adapter-family') in network_family:
            ports_res = self.adapter.get_property('network-port-uris')
        else:
            return port_list
        if ports_res:
            for port_uri in ports_res:
                port = Port(self, port_uri, {'element-uri': port_uri})
                if full_properties:
                    port.pull_full_properties()
                port_list.append(port)
        return port_list


class Port(BaseResource):
    """
    Representation of a :term:`Port`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    For the properties of a Port, see section
    'Data model - Port Element Object' in section 'Adapter object' in the
    :term:`HMC API` book.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.PortManager`).
    """

    def __init__(self, manager, uri, properties):
        # This function should not go into the docs.
        # Parameters:
        #   manager (:class:`~zhmcclient.PortManager`):
        #     Manager for this Port.
        #   uri (string):
        #     Canonical URI path of this Port.
        #   properties (dict):
        #     Properties to be set for this Port.
        #     See initialization of :class:`~zhmcclient.BaseResource` for
        #     details.
        assert isinstance(manager, PortManager)
        super(Port, self).__init__(manager, uri, properties)

    def update_properties(self, properties):
        """
        Update writeable properties of this Port.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model - Port Element Object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.post(self._uri, body=properties)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`Partition` is a subset of the hardware resources of a :term:`CPC`
in DPM mode, virtualized as a separate computer.

Partitions can be created and deleted dynamically, and their resources such
as CPU, memory or I/O devices can be configured dynamically.
You can create as many partition definitions as you want, but only a specific
number of partitions can be active at any given time.

TODO: How can a user find out what the maximum is, before it is reached?

Partition resources are contained in CPC resources.

Partition resources only exist in CPCs that are in DPM mode. CPCs in classic
mode (or ensemble mode) have :term:`LPAR` resources, instead.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource
from ._nic import NicManager
from ._hba import HbaManager
from ._virtual_function import VirtualFunctionManager
from ._logging import _log_call

__all__ = ['PartitionManager', 'Partition']


class PartitionManager(BaseManager):
    """
    Manager providing access to the :term:`Partitions <Partition>` in a
    particular :term:`CPC`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible as properties in higher level resources (in this case, the
    :class:`~zhmcclient.Cpc` object).
    """

    def __init__(self, cpc):
        # This function should not go into the docs.
        # Parameters:
        #   cpc (:class:`~zhmcclient.Cpc`):
        #     CPC defining the scope for this manager.
        super(PartitionManager, self).__init__(cpc)

    @property
    def cpc(self):
        """
        :class:`~zhmcclient.Cpc`: :term:`CPC` defining the scope for this
        manager.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        List the Partitions in this CPC.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.Partition` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        partitions_res = self.session.get(cpc_uri + '/partitions')
        partition_list = []
        if partitions_res:
            partition_items = partitions_res['partitions']
            for partition_props in partition_items:
                partition = Partition(self, partition_props['object-uri'],
                                      partition_props)
                if full_properties:
                    partition.pull_full_properties()
                partition_list.append(partition)
        return partition_list

    def create(self, properties):
        """
        Create and configure a Partition in this CPC.

        Parameters:

          properties (dict): Initial property values.
            Allowable properties are defined in section 'Request body contents'
            in section 'Create Partition' in the :term:`HMC API` book.

        Returns:

          Partition:
            The resource object for the new Partition.
            The object will have its 'object-uri' property set as returned by
            the HMC, and will also have the input properties set.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        result = self.session.post(cpc_uri + '/partitions', body=properties)
        # There should not be overlaps, but just in case there are, the
        # returned props should overwrite the input props:
        props = properties.copy()
        props.update(result)
        return Partition(self, props['object-uri'], props)

    def partition_object(self, part_id):
        """
        Return a minimalistic :class:`~zhmcclient.Partition` object for a
        Partition in this CPC.

        This method is an internal helper function and is not normally called
        by users.

        This object will be connected in the Python object tree representing
        the resources (i.e. it has this CPC as a parent), and will have the
        following properties set:

          * `object-uri`
          * `object-id`
          * `parent`
          * `class`

        Parameters:

            part_id (string): `object-id` of the Partition

        Returns:

            :class:`~zhmcclient.Partition`: A Python object representing the
            Partition.
        """
        part_uri = "/api/partitions/" + part_id
        return Partition(self, part_uri, {'object-uri': part_uri,
                                          'object-id': part_id,
                                          'parent': self.parent.uri,
                                          'class': 'partition'})


class Partition(BaseResource):
    """
    Representation of a :term:`Partition`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.PartitionManager`).
    """

    def __init__(self, manager, uri, properties):
        # This function should not go into the docs.
        # Parameters:
        #   manager (:class:`~zhmcclient.PartitionManager`):
        #     Manager for this Partition.
        #   uri (string):
        #     Canonical URI path of this Partition.
        #   properties (dict):
        #     Properties to be set for this Partition.
        #     See initialization of :class:`~zhmcclient.BaseResource` for
        #     details.
        assert isinstance(manager, PartitionManager)
        super(Partition, self).__init__(manager, uri, properties)
        self._nics = None
        self._hbas = None
        self._virtual_functions = None

    @property
    @_log_call
    def nics(self):
        """
        :class:`~zhmcclient.NicManager`: Access to the :term:`NICs <NIC>` in
        this Partition.
        """
        # We do here some lazy loading.
        if not self._nics:
            self._nics = NicManager(self)
        return self._nics

    @property
    @_log_call
    def hbas(self):
        """
        :class:`~zhmcclient.HbaManager`: Access to the :term:`HBAs <HBA>` in
        this Partition.
        """
        # We do here some lazy loading.
        if not self._hbas:
            self._hbas = HbaManager(self)
        return self._hbas

    @property
    @_log_call
    def virtual_functions(self):
        """
        :class:`~zhmcclient.VirtualFunctionManager`: Access to the
        :term:`Virtual Functions <Virtual Function>` in this Partition.
        """
        # We do here some lazy loading.
        if not self._virtual_functions:
            self._virtual_functions = VirtualFunctionManager(self)
        return self._virtual_functions

    def start(self, wait_for_completion=True):
        """
        Start (activate) this Partition, using the HMC operation "Start
        Partition".

        TODO: Describe what happens if the maximum number of active partitions
        is exceeded.

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            partition_uri + '/operations/start',
            wait_for_completion=wait_for_completion)
        return result

    def stop(self, wait_for_completion=True):
        """
        Stop (deactivate) this Partition, using the HMC operation "Stop
        Partition".

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            partition_uri + '/operations/stop',
            wait_for_completion=wait_for_completion)
        return result

    def delete(self):
        """
        Delete this Partition.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        self.manager.session.delete(partition_uri)

    def update_properties(self, properties):
        """
        Update writeable properties of this Partition.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model' in section 'Partition object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        self.manager.session.post(partition_uri, body=properties)

    def dump_partition(self, parameters, wait_for_completion=True):
        """
        Dump this Partition, by loading a standalone dump program from a SCSI
        device and starting its execution, using the HMC operation
        'Dump Partition'.

        Parameters:

          parameters (dict): Input parameters for the operation.
            Allowable input parameters are defined in section
            'Request body contents' in section 'Dump Partition' in the
            :term:`HMC API` book.

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            partition_uri + '/operations/scsi-dump',
            wait_for_completion=wait_for_completion, body=parameters)
        return result

    def psw_restart(self, wait_for_completion=True):
        """
        Initiates a PSW restart for this Partition, using the HMC operation
        'Perform PSW Restart'.

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            partition_uri + '/operations/psw-restart',
            wait_for_completion=wait_for_completion)
        return result

    def mount_iso_image(self, properties):
        """
        Upload an ISO image and associate it to this Partition
        using the HMC operation 'Mount ISO Image'.

        When the partition already has an ISO image associated,
        the newly uploaded image replaces the current one.

        TODO: The interface of this method has issues, see issue #57.

        Parameters:

          properties (dict): Properties for the dump operation.
            See the section in the :term:`HMC API` about the specific HMC
            operation and about the 'Mount ISO Image' description of the
            members of the passed properties dict.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        self.manager.session.post(
            partition_uri + '/operations/mount-iso-image',
            wait_for_completion=True, body=properties)

    def unmount_iso_image(self):
        """
        Unmount the currently mounted ISO from this Partition using the HMC
        operation 'Unmount ISO Image'.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        self.manager.session.post(
            partition_uri + '/operations/unmount-iso-image')



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`Virtual Switch` is a virtualized networking switch connecting
:term:`NICs <NIC>` with a :term:`Network Port`.

Virtual Switches are generated automatically every time a new
:term:`Network Adapter` is detected and configured.

Virtual Switch resources are contained in :term:`CPC` resources.

Virtual Switches only exist in CPCs that are in DPM mode.
"""

from __future__ import absolute_import

import re

from ._manager import BaseManager
from ._resource import BaseResource

__all__ = ['VirtualSwitchManager', 'VirtualSwitch']


class VirtualSwitchManager(BaseManager):
    """
    Manager providing access to the :term:`Virtual Switches <Virtual Switch>`
    in a particular :term:`CPC`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible as properties in higher level resources (in this case, the
    :class:`~zhmcclient.Cpc` object).
    """

    def __init__(self, cpc):
        # This function should not go into the docs.
        # Parameters:
        #   cpc (:class:`~zhmcclient.Cpc`):
        #     CPC defining the scope for this manager.
        super(VirtualSwitchManager, self).__init__(cpc)

    @property
    def cpc(self):
        """
        :class:`~zhmcclient.Cpc`: :term:`CPC` defining the scope for this
        manager.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        List the Virtual Switches in this CPC.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.VirtualSwitch` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        vswitch_res = self.session.get(cpc_uri + '/virtual-switches')
        vswitch_list = []
        if vswitch_res:
            vswitch_items = vswitch_res['virtual-switches']
            for vswitch_props in vswitch_items:
                vswitch = VirtualSwitch(self, vswitch_props['object-uri'],
                                        vswitch_props)
                if full_properties:
                    vswitch.pull_full_properties()
                vswitch_list.append(vswitch)
        return vswitch_list


class VirtualSwitch(BaseResource):
    """
    Representation of a :term:`Virtual Switch`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    For the properties of a Virtual Switch, see section 'Data model'
    in section 'Virtual Switch object' in the :term:`HMC API` book.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.VirtualSwitchManager`).
    """

    def __init__(self, manager, uri, properties):
        # This function should not go into the docs.
        # Parameters:
        #   manager (:class:`~zhmcclient.VirtualSwitchManager`):
        #     Manager object for this Virtual Switch.
        #   uri (string):
        #     Canonical URI path of this Virtual Switch.
        #   properties (dict):
        #     Properties to be set for this Virtual Switch.
        #     See initialization of :class:`~zhmcclient.BaseResource` for
        #     details.
        assert isinstance(manager, VirtualSwitchManager)
        super(VirtualSwitch, self).__init__(manager, uri, properties)

    def get_connected_nics(self):
        """
        List the :term:`NICs <NIC>` connected to this Virtual Switch.

        This method performs the "Get Connected VNICs of a Virtual Switch" HMC
        operation.

        Returns:

          : A list of :term:`Nic` objects. These objects will be connected in
          the resource tree (i.e. have a parent :term:`Partition` object,
          etc.) and will have the following properties set:

          * `element-uri`
          * `element-id`
          * `parent`
          * `class`

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        vswitch_uri = self.get_property('object-uri')
        result = self.manager.session.get(
            vswitch_uri + '/operations/get-connected-vnics')
        nic_uris = result['connected-vnic-uris']
        nic_list = []
        parts = {}  # Key: Partition ID; Value: Partition object
        for nic_uri in nic_uris:
            m = re.match(r"^/api/partitions/([^/]+)/nics/([^/]+)/?$", nic_uri)
            part_id = m.group(1)
            nic_id = m.group(2)
            # We remember created Partition objects and reuse them.
            try:
                part = parts[part_id]
            except KeyError:
                part = self.manager.cpc.partitions.partition_object(part_id)
                parts[part_id] = part
            nic = part.nics.nic_object(nic_id)
            nic_list.append(nic)
        return nic_list

    def update_properties(self, properties):
        """
        Update writeable properties of this Virtual Switch.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model' in section 'Virtual Switch object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        vswitch_uri = self.get_property('object-uri')
        self.manager.session.post(vswitch_uri, body=properties)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`HBA` (Host Bus Adapter) is a logical entity that provides a
:term:`Partition` with access to external storage area networks (SANs) through
an :term:`FCP Adapter`. More specifically, an HBA connects a Partition with an
:term:`Adapter Port` on an FCP Adapter.

HBA resources are contained in Partition resources.

HBAs only exist in :term:`CPCs <CPC>` that are in DPM mode.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource

__all__ = ['HbaManager', 'Hba']


class HbaManager(BaseManager):
    """
    Manager providing access to the :term:`HBAs <HBA>` in a particular
    :term:`Partition`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible as properties in higher level resources (in this case, the
    :class:`~zhmcclient.Partition` object).
    """

    def __init__(self, partition):
        # This function should not go into the docs.
        # Parameters:
        #   partition (:class:`~zhmcclient.Partition`):
        #     Partition defining the scope for this manager.
        super(HbaManager, self).__init__(partition)

    @property
    def partition(self):
        """
        :class:`~zhmcclient.Partition`: :term:`Partition` defining the scope
        for this manager.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        List the HBAs in this Partition.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.Hba` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        hbas_res = self.partition.get_property('hba-uris')
        hba_list = []
        if hbas_res:
            for hba_uri in hbas_res:
                hba = Hba(self, hba_uri, {'element-uri': hba_uri})
                if full_properties:
                    hba.pull_full_properties()
                hba_list.append(hba)
        return hba_list

    def create(self, properties):
        """
        Create and configure an HBA in this Partition.

        Parameters:

          properties (dict): Initial property values.
            Allowable properties are defined in section 'Request body contents'
            in section 'Create HBA' in the :term:`HMC API` book.

            The underlying :term:`FCP port` for the new HBA is assigned via the
            "adapter-port-uri" property.

        Returns:

          Hba:
            The resource object for the new HBA.
            The object will have its 'element-uri' property set as returned by
            the HMC, and will also have the input properties set.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.partition.get_property('object-uri')
        result = self.session.post(partition_uri + '/hbas', body=properties)
        # There should not be overlaps, but just in case there are, the
        # returned props should overwrite the input props:
        props = properties.copy()
        props.update(result)
        return Hba(self, props['element-uri'], props)


class Hba(BaseResource):
    """
    Representation of an :term:`HBA`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    For the properties of an HBA resource, see section
    'Data model - HBA Element Object' in section 'Partition object' in the
    :term:`HMC API` book.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.HbaManager`).
    """

    def __init__(self, manager, uri, properties):
        # This function should not go into the docs.
        # Parameters:
        #   manager (:class:`~zhmcclient.HbaManager`):
        #     Manager for this HBA.
        #   uri (string):
        #     Canonical URI path of this HBA.
        #   properties (dict):
        #     Properties to be set for this HBA.
        #     See initialization of :class:`~zhmcclient.BaseResource` for
        #     details.
        assert isinstance(manager, HbaManager)
        super(Hba, self).__init__(manager, uri, properties)

    def delete(self):
        """
        Delete this HBA.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.delete(self._uri)

    def update_properties(self, properties):
        """
        Update writeable properties of this HBA.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model - HBA Element Object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.post(self._uri, body=properties)

    def reassign_port(self, port):
        """
        Reassign this HBA to a new underlying :term:`FCP port`.

        This method performs the HMC operation "Reassign Storage Adapter Port".

        Parameters:

          port (:class:`~zhmcclient.Port`): :term:`FCP port` to be used.

        Raises:

          :exc:`~zhmcclient.HTTPError`: See the HTTP status and reason codes of
            operation "Reassign Storage Adapter Port" in the :term:`HMC API`
            book.
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        body = {'adapter-port-uri': port.uri}
        self.manager.session.post(self._uri +
                                  '/operations/reassign-storage-adapter-port',
                                  body=body)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import click

import zhmcclient
from .zhmccli import cli
from ._helper import print_properties, print_resources


@cli.group('cpc')
def cpc_group():
    """Command group for managing CPCs."""


@cpc_group.command('list')
@click.pass_obj
def cpc_list(cmd_ctx):
    """List the CPCs."""
    cmd_ctx.execute_cmd(lambda: cmd_cpc_list(cmd_ctx))


@cpc_group.command('show')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.pass_obj
def cpc_show(cmd_ctx, cpc_name):
    """Show details of a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_cpc_show(cmd_ctx, cpc_name))


def cmd_cpc_list(cmd_ctx):
    """
    List CPCs.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    try:
        cpcs = client.cpcs.list()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    print_resources(cpcs, cmd_ctx.output_format)


def cmd_cpc_show(cmd_ctx, cpc_name):
    """
    Show details of a CPC. In table format, some properties are skipped.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    try:
        cpc = client.cpcs.find(name=cpc_name)
        cpc.pull_full_properties()
    except zhmcclient.NotFound:
        raise click.ClickException("Could not find CPC %s on HMC %s" %
                                   (cpc_name, cmd_ctx.session.host))
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    skip_list = ('ec-mcl-description',
                 'cpc-power-saving-state',
                 '@@implementation-errors',
                 'network2-ipv6-info',
                 'network1-ipv6-info',
                 'auto-start-list')
    print_properties(cpc.properties, cmd_ctx.output_format, skip_list)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import click

import zhmcclient
from .zhmccli import cli
from ._helper import print_properties, print_resources, abort_if_false, \
    options_to_properties


@cli.group('partition')
def partition_group():
    """Command group for managing partitions."""


@partition_group.command('list')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.pass_obj
def partition_list(cmd_ctx, cpc_name):
    """List the partitions in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_partition_list(cmd_ctx, cpc_name))


@partition_group.command('create')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.option('--name', type=str, required=True,
              help='The name of the partition.')
@click.option('--description', type=str, required=False,
              help='The description associated with this partition.')
@click.option('--cp-processors', type=int, required=True,
              help='Defines the number of general purpose processors (CP).')
@click.option('--initial-memory', type=int, required=True,
              help='The initial amount of memory when the partition is '
                   'started.')
@click.option('--maximum-memory', type=int, required=True,
              help='The maximum size while the partition is running.')
@click.option('--processor-mode', type=str, required=True,
              help='Defines how processors are allocated to the partition.')
@click.option('--boot-device', type=str, required=True,
              help='The type of device from which the partition is booted.')
@click.pass_context
def partition_create(ctx, cpc_name, **options):
    """Create a partition in a CPC."""
    properties = options_to_properties(options)
    cmd_ctx = ctx.obj
    cmd_ctx.execute_cmd(lambda: cmd_partition_create(cmd_ctx, cpc_name,
                                                     properties))


@partition_group.command('show')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('PARTITION-NAME', type=str, metavar='PARTITION-NAME')
@click.pass_obj
def partition_show(cmd_ctx, cpc_name, partition_name):
    """Show the details of a partition in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_partition_show(cmd_ctx, cpc_name,
                                                   partition_name))


@partition_group.command('start')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('PARTITION-NAME', type=str, metavar='PARTITION-NAME')
@click.pass_obj
def partition_start(cmd_ctx, cpc_name, partition_name):
    """Start a partition in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_partition_start(cmd_ctx, cpc_name,
                                                    partition_name))


@partition_group.command('stop')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('PARTITION-NAME', type=str, metavar='PARTITION-NAME')
@click.pass_obj
def partition_stop(cmd_ctx, cpc_name, partition_name):
    """Stop a partition in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_partition_stop(cmd_ctx, cpc_name,
                                                   partition_name))


@partition_group.command('delete')
@click.argument('CPC-NAME', type=str, metavar='CPC-NAME')
@click.argument('PARTITION-NAME', type=str, metavar='PARTITION-NAME')
@click.option('--yes', is_flag=True, callback=abort_if_false,
              expose_value=False,
              help='Skip prompt to confirm deletion of the partition.',
              prompt='Are you sure you want to delete this partition ?')
@click.pass_obj
def partition_delete(cmd_ctx, cpc_name, partition_name):
    """Delete a partition in a CPC."""
    cmd_ctx.execute_cmd(lambda: cmd_partition_delete(cmd_ctx, cpc_name,
                                                     partition_name))


def _find_cpc(client, cpc_name):
    try:
        cpc = client.cpcs.find(name=cpc_name)
    except zhmcclient.NotFound:
        raise click.ClickException("Could not find CPC %s on HMC %s." %
                                   (cpc_name, client.session.host))
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    return cpc


def _find_partition(client, cpc_name, partition_name):
    cpc = _find_cpc(client, cpc_name)
    if not cpc.dpm_enabled:
        raise click.ClickException("CPC %s is not in DPM mode." % cpc_name)
    try:
        partition = cpc.partitions.find(name=partition_name)
    except zhmcclient.NotFound:
        raise click.ClickException("Could not find partition %s in CPC %s." %
                                   (partition_name, cpc_name))
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    return partition


def cmd_partition_list(cmd_ctx, cpc_name):
    """
    List the partitions in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    cpc = _find_cpc(client, cpc_name)
    if not cpc.dpm_enabled:
        raise click.ClickException("CPC %s is not in DPM mode." % cpc_name)
    try:
        partitions = cpc.partitions.list()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    print_resources(partitions, cmd_ctx.output_format)


def cmd_partition_show(cmd_ctx, cpc_name, partition_name):
    """
    Show the details of a partition in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    partition = _find_partition(client, cpc_name, partition_name)
    try:
        partition.pull_full_properties()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    skip_list = ('nic-uris')
    print_properties(partition.properties, cmd_ctx.output_format, skip_list)


def cmd_partition_start(cmd_ctx, cpc_name, partition_name):
    """
    Start a partition in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    partition = _find_partition(client, cpc_name, partition_name)
    try:
        partition.start(wait_for_completion=True)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    click.echo('Partition %s has been started.' % partition_name)


def cmd_partition_stop(cmd_ctx, cpc_name, partition_name):
    """
    Stop a partition in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    partition = _find_partition(client, cpc_name, partition_name)
    try:
        partition.stop(wait_for_completion=True)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    click.echo('Partition %s has been stopped.' % partition_name)


def cmd_partition_create(cmd_ctx, cpc_name, properties):
    """
    Create a new partition in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    cpc = _find_cpc(client, cpc_name)
    if not cpc.dpm_enabled:
        raise click.ClickException("CPC %s is not in DPM mode." % cpc_name)
    try:
        new_partition = cpc.partitions.create(properties)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    click.echo("New partition %s has been created." %
               new_partition.properties['name'])


def cmd_partition_delete(cmd_ctx, cpc_name, partition_name):
    """
    Delete a partition in a CPC.
    """
    client = zhmcclient.Client(cmd_ctx.session)
    partition = _find_partition(client, cpc_name, partition_name)
    try:
        partition.delete()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    click.echo('Partition %s has been deleted.' % partition_name)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`Virtual Function` is a logical entity that provides a
:term:`Partition` with access to
:term:`Accelerator Adapters <Accelerator Adapter>`.

Virtual Function resources are contained in Partition resources.

Virtual Functions only exist in :term:`CPCs <CPC>` that are in DPM mode.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource

__all__ = ['VirtualFunctionManager', 'VirtualFunction']


class VirtualFunctionManager(BaseManager):
    """
    Manager providing access to the
    :term:`Virtual Functions <Virtual Function>` in a particular
    :term:`Partition`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible as properties in higher level resources (in this case, the
    :class:`~zhmcclient.Partition` object).
    """

    def __init__(self, partition):
        # This function should not go into the docs.
        # Parameters:
        #   partition (:class:`~zhmcclient.Partition`):
        #     Partition defining the scope for this manager.
        super(VirtualFunctionManager, self).__init__(partition)

    @property
    def partition(self):
        """
        :class:`~zhmcclient.Partition`: :term:`Partition` defining the scope
        for this manager.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        List the Virtual Functions of this Partition.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.VirtualFunction` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        vfs_res = self.partition.get_property('virtual-function-uris')
        vf_list = []
        if vfs_res:
            for vf_uri in vfs_res:
                vf = VirtualFunction(self, vf_uri, {'element-uri': vf_uri})
                if full_properties:
                    vf.pull_full_properties()
                vf_list.append(vf)
        return vf_list

    def create(self, properties):
        """
        Create a Virtual Function in this Partition.

        Parameters:

          properties (dict): Initial property values.
            Allowable properties are defined in section 'Request body contents'
            in section 'Create Virtual Function' in the :term:`HMC API` book.

        Returns:

          VirtualFunction:
            The resource object for the new Virtual Function.
            The object will have its 'element-uri' property set as returned by
            the HMC, and will also have the input properties set.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.partition.get_property('object-uri')
        result = self.session.post(partition_uri + '/virtual-functions',
                                   body=properties)
        # There should not be overlaps, but just in case there are, the
        # returned props should overwrite the input props:
        props = properties.copy()
        props.update(result)
        return VirtualFunction(self, props['element-uri'], props)


class VirtualFunction(BaseResource):
    """
    Representation of a :term:`Virtual Function`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    For the properties of a Virtual Function, see section
    'Data model - Virtual Function Element Object' in section
    'Partition object' in the :term:`HMC API` book.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.VirtualFunctionManager`).
    """

    def __init__(self, manager, uri, properties):
        # This function should not go into the docs.
        # Parameters:
        #   manager (:class:`~zhmcclient.VirtualFunctionManager`):
        #     Manager object for this Virtual Function.
        #   uri (string):
        #     Canonical URI path of this Virtual Function.
        #   properties (dict):
        #     Properties to be set for this Virtual Function.
        #     See initialization of :class:`~zhmcclient.BaseResource` for
        #     details.
        assert isinstance(manager, VirtualFunctionManager)
        super(VirtualFunction, self).__init__(manager, uri, properties)

    def delete(self):
        """
        Delete this Virtual Function.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.delete(self._uri)

    def update_properties(self, properties):
        """
        Update writeable properties of this Virtual Function.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model - Virtual Function element object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.post(self._uri, body=properties)



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import os
import requests.packages.urllib3
import click
from click_repl import register_repl, repl

from ._helper import CmdContext


# TODO: Documentation for zhmc cli
# TODO: Automated tests for zhmc cli
# TODO: Find a way to clarify in the sub-command help texts that the global
#       options can be used.


requests.packages.urllib3.disable_warnings()

# Default values for some options
DEFAULT_OUTPUT_FORMAT = 'table'
DEFAULT_TIMESTATS = False


@click.group(invoke_without_command=True)
@click.option('-h', '--host', type=str, envvar='ZHMC_HOST',
              help="Hostname or IP address of the HMC "
                   "(Default: ZHMC_HOST environment variable).")
@click.option('-u', '--userid', type=str, envvar='ZHMC_USERID',
              help="Username for the HMC "
                   "(Default: ZHMC_USERID environment variable).")
@click.option('-o', '--output-format', type=click.Choice(['table', 'json']),
              help='Output format (Default: {of}).'
              .format(of=DEFAULT_OUTPUT_FORMAT))
@click.option('-t', '--timestats', type=str, is_flag=True,
              help='Show time statistics of HMC operations.')
@click.version_option(help="Show the version of this command and exit.")
@click.pass_context
def cli(ctx, host, userid, output_format, timestats):
    """
    Command line interface for the z Systems HMC.

    zhmc is a command line interface for interacting with the Web Services API
    of the Hardware Management Console (HMC) of z Systems or LinuxONE machines.
    It allows management of LPARs, partitions and much more.

    \b
    The zhmc command line interface can be used in these modes:
    - Interactive - an interactive shell
    - Command-Session - command mode that prompts for a password only once and
      stores the session-id in an environment variable
    - Command-Prompt - command mode that prompts for a password every time

    Example for interactive mode:

       \b
       $ zhmc -h zhmc.example.com -u hmcuser
       Enter password: <password>
       > cpc list
       > partition list JLSE1
       > partition create JLSE1 --name TESTPART_JL --description "blablabla"
       --cp-processors 1 --initial-memory 4096 --maximum-memory 4096
       --processor-mode shared --boot-device test-operating-system
       > partition show JLSE1 TESTPART_JL
       > partition start JLSE1 TESTPART_JL
       > partition stop JLSE1 TESTPART_JL
       > partition delete JLSE1 TESTPART_JL --yes
       > :q

    Example for command-session mode:

       \b
       $ eval $(zhmc -h zhmc.example.com -u hmcuser session create)
       Enter password: <password>
       $ zhmc cpc list
       $ zhmc lpar list P0000P99
       $ zhmc lpar deactivate P0000P99 PART8 --yes
       $ zhmc --timestats lpar activate P0000P99 PART8
       $ zhmc lpar load P0000P99 PART8 5172
       $ zhmc lpar show P0000P99 PART8
       $ zhmc -o json lpar show P0000P30 PART8
       $ eval $(zhmc session delete --yes)

    Example for command-prompt mode:

       \b
       $ zhmc -h zhmc.example.com -u hmcuser cpc list
       Enter password: <password>
       $ zhmc -h zhmc.example.com info
       Enter password: <password>

    To get overall help:

       \b
       $ zhmc --help

    To get help for a specific command (or command group):

       \b
       $ zhmc <command> --help

    To enable bash completion in the current shell:

       \b
       $ eval "$(_ZHMC_COMPLETE=source zhmc)"

    To use bash completion:

       \b
       $ zhmc --<TAB><TAB>
       ... shows the global options to select from ...
       $ zhmc <TAB><TAB>
       ... shows the commands to select from ...
    """

    # Concept: In interactive mode, the global options specified in the command
    # line are used as defaults for the commands that are issued interactively.
    # The interactive commands may override these options.
    # This requires being able to determine for each option whether it has been
    # specified. This is the reason the options don't define defaults in the
    # decorators that define them.

    if ctx.obj is None:
        # We are in command mode or are processing the command line options in
        # interactive mode.
        # We apply the documented option defaults.
        if output_format is None:
            output_format = DEFAULT_OUTPUT_FORMAT
        if timestats is None:
            timestats = DEFAULT_TIMESTATS
    else:
        # We are processing an interactive command.
        # We apply the option defaults from the command line options.
        if host is None:
            host = ctx.obj.host
        if userid is None:
            userid = ctx.obj.userid
        if output_format is None:
            output_format = ctx.obj.output_format
        if timestats is None:
            timestats = ctx.obj.timestats

    # Now we have the effective values for the options as they should be used
    # by the current command, regardless of the mode.

    session_id = os.environ.get('ZHMC_SESSION_ID', None)

    def password_prompt():
        if userid is not None and host is not None:
            spinner_running = (ctx.obj.spinner is not None)
            if spinner_running:
                ctx.obj.spinner_stop()
            password = click.prompt(
                "Enter password (for user {userid} at HMC {host})"
                .format(userid=userid, host=host), hide_input=True,
                confirmation_prompt=False, type=str, err=True)
            if spinner_running:
                ctx.obj.spinner_start()
            return password
        else:
            raise click.ClickException("{cmd} command requires logon, but "
                                       "no session-id or userid provided."
                                       .format(cmd=ctx.invoked_subcommand))

    # We create a command context for each command: An interactive command has
    # its own command context different from the command context for the
    # command line.
    ctx.obj = CmdContext(host, userid, output_format, timestats, session_id,
                         password_prompt)

    # Invoke default command
    if ctx.invoked_subcommand is None:
        repl(ctx)


register_repl(cli)



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example 7: Demonstrate JMS notifications for completion of async operation
"""

import sys
from time import sleep
import threading
from pprint import pprint
import yaml
import requests.packages.urllib3
import stomp


import zhmcclient
__callback = None

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" %
          (hmccreds_file))
    sys.exit(1)

example7 = examples.get("example7", None)
if example7 is None:
    print("example7 not found in credentials file %s" %
          (hmccreds_file))
    sys.exit(1)

hmc = example7["hmc"]
cpcname = example7["cpcname"]
partname = example7["partname"]
amqport = example7['amqport']
callback = None
topic = None

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" %
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

# Thread-safe handover of notifications between listener and main threads
NOTI_DATA = None
NOTI_LOCK = threading.Condition()

class MyListener(object):

    def on_connecting(self, host_and_port):
        print("Listener: Attempting to connect to message broker")
        sys.stdout.flush()

    def on_connected(self, headers, message):
        print("Listener: Connected to broker")
        sys.stdout.flush()

    def on_disconnected(self):
        print("Listener: Disconnected from broker")
        sys.stdout.flush()

    def on_error(self, headers, message):
        print('Listener: Received an error: %s' % message)
        sys.stdout.flush()

    def on_message(self, headers, message):
        global NOTI_DATA, NOTI_LOCK
        print('Listener: Received a notification')
        sys.stdout.flush()
        with NOTI_LOCK:
            # Wait until main program has processed the previous notification
            while NOTI_DATA:
                NOTI_LOCK.wait()
            # Indicate to main program that there is a new notification
            NOTI_DATA = headers
            NOTI_LOCK.notifyAll()

try:
    print("Using HMC %s with userid %s ..." % (hmc, userid))
    session = zhmcclient.Session(hmc, userid, password)
    cl = zhmcclient.Client(session)

    print("Retrieving notification topics ...")
    topics = session.get_notification_topics()

    for topic in topics:
        if topic['topic-type'] == 'job-notification':
            job_topic_name = topic['topic-name']
            break

    conn = stomp.Connection([(session.host, amqport)], use_ssl="SSL")
    conn.set_listener('', MyListener())
    conn.start()
    conn.connect(userid, password, wait=True)

    sub_id = 42  # subscription ID

    print("Subscribing for job notifications using topic: %s" % job_topic_name)
    conn.subscribe(destination="/topic/"+job_topic_name, id=sub_id, ack='auto')

    print("Finding CPC %s ..." % cpcname)
    cpc = cl.cpcs.find(name=cpcname)
    print("Status of CPC %s: %s" % (cpcname, cpc.get_property('status')))

    print("Finding partition %s ..." % partname)
    partition = cpc.partitions.find(name=partname)
    partition_status = partition.get_property('status')
    print("Status of partition %s: %s" % (partname, partition_status))

    if partition_status == 'active':
        print("Stopping partition %s asynchronously ..." % partname)
        result = partition.stop(wait_for_completion=False)
    elif partition_status in ('inactive', 'stopped'):
        print("Starting partition %s asynchronously ..." % partname)
        result = partition.start(wait_for_completion=False)
    else:
        raise zhmcclient.Error("Cannot deal with partition status: %s" % \
                               partition_status)

    job_uri = result['job-uri']
    print("Waiting for completion of job %s ..." % job_uri)
    sys.stdout.flush()

    # Just for demo purposes, we show how a loop for processing multiple
    # notifications would look like.
    while True:
        with NOTI_LOCK:

            # Wait until listener has a new notification
            while not NOTI_DATA:
                NOTI_LOCK.wait()

            # Process the notification
            print("Received notification:")
            pprint(NOTI_DATA)
            sys.stdout.flush()

            # This test is just for demo purposes, it should always be our job
            # given what we subscribed for.
            if NOTI_DATA['job-uri'] == job_uri:
                break
            else:
                print("Unexpected completion received for job %s" % \
                      NOTI_DATA['job-uri'])
                sys.stdout.flush()

            # Indicate to listener that we are ready for next notification
            NOTI_DATA = None
            NOTI_LOCK.notifyAll()

    job_uri = NOTI_DATA['job-uri']
    print("Job has completed: %s" % job_uri)
    sys.stdout.flush()

    conn.disconnect()
    sleep(1)  # Allow listener to print disconnect message (just for demo)

except zhmcclient.Error as exc:
    print("%s: %s" % (exc.__class__.__name__, exc))
    sys.exit(1)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import sys
import json
import click
import click_spinner
from tabulate import tabulate

import zhmcclient


def abort_if_false(ctx, param, value):
    # pylint: disable=unused-argument
    if not value:
        raise click.ClickException("Aborted.")


class InvalidOutputFormatError(click.ClickException):
    """
    Exception indicating an invalid output format for zhmc.
    """

    def __init__(self, output_format):
        msg = "Invalid output format: {of}".format(of=output_format)
        super(InvalidOutputFormatError, self).__init__(msg)


class CmdContext(object):

    def __init__(self, host, userid, output_format, timestats, session_id,
                 get_password):
        self._host = host
        self._userid = userid
        self._output_format = output_format
        self._timestats = timestats
        self._session_id = session_id
        self._get_password = get_password
        self._session = None
        self._spinner = None

    @property
    def host(self):
        """
        :term:`string`: Hostname or IP address of the HMC.
        """
        return self._host

    @property
    def userid(self):
        """
        :term:`string`: Userid on the HMC.
        """
        return self._userid

    @property
    def output_format(self):
        """
        :term:`string`: Output format to be used.
        """
        return self._output_format

    @property
    def timestats(self):
        """
        bool: Indicates whether time statistics should be printed.
        """
        return self._timestats

    @property
    def session_id(self):
        """
        :term:`string`: Session-id to be used instead of logging on, or `None`.
        """
        return self._session_id

    @property
    def get_password(self):
        """
        :term:`callable`: Password retrieval function, or `None`.
        """
        return self._get_password

    @property
    def session(self):
        """
        :class:`requests.Session` object once logged on, or `None`.
        """
        return self._session

    @property
    def spinner(self):
        """
        :class:`~click_spinner.Spinner` object if a spinner is running, or
        `None`.
        """
        return self._spinner

    def spinner_start(self):
        """
        Start the spinner (unless redirected).

        Unlike :meth:`click_spinner.Spinner.start`, this function can be called
        to restart the spinner after having stopped it (via
        :meth:`spinner_stop`).
        """
        if sys.stdout.isatty():
            self._spinner = click_spinner.Spinner()
            self._spinner.start()

    def spinner_stop(self):
        """
        Stop the spinner.
        """
        if self._spinner is not None:
            self._spinner.stop()
            self._spinner = None

    def execute_cmd(self, cmd):
        if self._session is None:
            if self._host is None:
                raise click.ClickException("No HMC host provided")
            if self._session_id is not None:
                self._session = zhmcclient.Session(
                    self._host, self._userid, session_id=self._session_id,
                    get_password=self._get_password)
            else:
                self._session = zhmcclient.Session(
                    self._host, self._userid, get_password=self._get_password)
        if self.timestats:
            self._session.time_stats_keeper.enable()
        self.spinner_start()
        try:
            cmd()
        finally:
            self.spinner_stop()
            if self._session.time_stats_keeper.enabled:
                click.echo(self._session.time_stats_keeper)


def options_to_properties(options):
    properties = {}
    for k, v in options.iteritems():
        properties[k.replace('_', '-')] = v
    return properties


def print_properties(properties, output_format, skip_list=None):
    """
    Print properties in the desired output format.
    """
    if output_format == 'table':
        print_properties_as_table(properties, skip_list)
    elif output_format == 'json':
        print_properties_as_json(properties)
    else:
        raise InvalidOutputFormatError(output_format)


def print_resources(resources, output_format):
    """
    Print the properties of a list of resources in the desired output format.
    """
    if output_format == 'table':
        print_resources_as_table(resources)
    elif output_format == 'json':
        print_resources_as_json(resources)
    else:
        raise InvalidOutputFormatError(output_format)


def print_properties_as_table(properties, skip_list=None):
    """
    Print properties in tabular output format.
    """
    table = list()
    sorted_fields = sorted(properties)
    for field in sorted_fields:
        if skip_list and field in skip_list:
            continue
        value = properties[field]
        table.append((field, value))
    headers = ['Field Name', 'Value']
    click.echo(tabulate(table, headers, tablefmt="psql"))


def print_resources_as_table(resources):
    """
    Print list of resources in tabular output format.
    """
    table = list()
    for i, resource in enumerate(resources):
        if i == 0:
            headers = resource.properties.keys()
        row = resource.properties.values()
        table.append(reversed(row))
    if not table:
        click.echo("No entries.")
    else:
        click.echo(tabulate(table, reversed(headers), tablefmt="psql"))


def print_properties_as_json(properties):
    """
    Print properties in JSON output format.
    """
    json_str = json.dumps(properties)
    click.echo(json_str)


def print_resources_as_json(resources):
    """
    Print properties of a list of resources in JSON output format.
    """
    json_str = json.dumps([r.properties for r in resources])
    click.echo(json_str)
